{*****************************************************************************}
{  DDObjects                                            www.delphi-online.at  }
{  Copyright (C) 2006 Stefan Meisner, stefan.meisner@gmx.net                  }
{  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS    }
{  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABI-    }
{  LITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY  }
{  RIGHTS. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR   }
{  ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,   }
{  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFT-    }
{  WARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         }
{*****************************************************************************}

unit uClientMainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, Spin, DDOClient, uDemoClass_Proxy, DDOBase;

type
  TClientMainForm = class(TForm)
    EditPort: TSpinEdit;
    Label1: TLabel;
    EditHost: TEdit;
    Label2: TLabel;
    ButtonGetQuote: TButton;
    MemoResult: TMemo;
    DDORequester1: TDDORequester;
    ButtonGetTime: TButton;
    ButtonStressTest: TButton;
    ButtonGetVar: TButton;
    ButtonPerform: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure EditHostChange(Sender: TObject);
    procedure EditPortChange(Sender: TObject);
    procedure ButtonGetQuoteClick(Sender: TObject);
    procedure ButtonGetTimeClick(Sender: TObject);
    procedure ButtonStressTestClick(Sender: TObject);
    procedure ButtonGetVarClick(Sender: TObject);
    procedure ButtonPerformClick(Sender: TObject);
  private
    FDemoClass: TDemoClass;
    FThreadCount: Integer;
    procedure ThreadTerminated(Sender: TObject);
    procedure UpdateGUI;
  end;

var
  ClientMainForm: TClientMainForm;

implementation

uses
  DDOUtils;

{$R *.dfm}

const
  CClientCount = 25;
  CClientCalls = 200;
  CPerformCount = 2500;

type
  TDemoClientThread = class(TThread)
  private
    FDemoClass: TDemoClass;
    FException: Exception;
  protected
    property DemoClass: TDemoClass read FDemoClass;
    procedure Execute; override;
  public
    constructor Create(ARequester: TDDORequester); reintroduce;
    destructor Destroy; override;
  end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

constructor TDemoClientThread.Create(ARequester: TDDORequester);
begin
  inherited Create(True);
  FDemoClass := TDemoClass.Create(
    TDDORequester.CreateCopy(ARequester), True);
  FreeOnTerminate := True;
  Resume;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

destructor TDemoClientThread.Destroy;
begin
  FreeAndNil(FDemoClass);
  inherited;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TDemoClientThread.Execute;
var
  i: Integer;
  DateTime: TDateTime;
begin
  try
    for i := 1 to CClientCalls do
      FDemoClass.GetTime(DateTime);
  except
    Terminate;
  end;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TClientMainForm.FormCreate(Sender: TObject);
begin
  EditHost.Text := DDORequester1.Host;
  EditPort.Value := DDORequester1.Port;
  FDemoClass := TDemoClass.Create(DDORequester1);
  UpdateGUI;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TClientMainForm.FormDestroy(Sender: TObject);
begin
  FreeAndNil(FDemoClass);
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TClientMainForm.EditHostChange(Sender: TObject);
begin
  DDORequester1.Host := EditHost.Text;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TClientMainForm.EditPortChange(Sender: TObject);
begin
  DDORequester1.Port := EditPort.Value;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TClientMainForm.ButtonGetQuoteClick(Sender: TObject);
var
  AQuote: String;
begin
  FDemoClass.GetQuote(AQuote);
  MemoResult.Text := AQuote;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TClientMainForm.ButtonGetTimeClick(Sender: TObject);
var
  DateTime: TDateTime;
begin
  FDemoClass.GetTime(DateTime);
  MemoResult.Text := DateTimeToStr(DateTime);
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TClientMainForm.ButtonGetVarClick(Sender: TObject);
var
  AString: String;
begin
  AString := EmptyStr;
  if InputQuery('var parameter', 'Value:', AString) then
  begin
    FDemoClass.GetVarParam(AString);
    MemoResult.Text := AString;
  end;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TClientMainForm.ButtonStressTestClick(Sender: TObject);
var
  i: Integer;
  lThread: TThread;
begin
  if MessageDlg(Format('This will spawn %d threads' + #13#10
    + 'executing %d calls each', [CClientCount, CClientCalls]),
    mtWarning, [mbOK,mbCancel], 0) = mrOk then
  begin
    for i := 1 to CClientCount do
    begin
      lThread := TDemoClientThread.Create(DDORequester1);
      lThread.OnTerminate := ThreadTerminated;
      InterlockedIncrement(FThreadCount);
      UpdateGUI;
    end;
  end;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TClientMainForm.ThreadTerminated(Sender: TObject);
begin
  InterlockedDecrement(FThreadCount);
  UpdateGUI;
end;

//----------------------------------------------------------------------------
//2006-13-02 N,SM:

procedure TClientMainForm.UpdateGUI;
begin
  Caption := Format('Client (%d threads)', [FThreadCount]);
  ButtonStressTest.Enabled := FThreadCount = 0;
  ButtonPerform.Enabled := FThreadCount = 0;
end;

//----------------------------------------------------------------------------
//2006-02-26 N,SM:

procedure TClientMainForm.ButtonPerformClick(Sender: TObject);
var
  i: Integer;
  lDateTime: TDateTime;
  lTimeStamp: Cardinal;
begin
  if MessageDlg(Format('This will execute %d calls',
    [CPerformCount]), mtWarning, [mbOK,mbCancel], 0) = mrOk then
  begin
    MemoResult.Clear;
    lTimeStamp := MeasureTime(0);
    for i := 1 to CPerformCount do
      FDemoClass.GetTime(lDateTime);
    lTimeStamp := MeasureTime(lTimeStamp);
    MemoResult.Lines.Add(IntToStr(lTimeStamp) + ' MSecs.');
    MemoResult.Lines.Add(IntToStr(Trunc(CPerformCount / (lTimeStamp / 1000)))
      + ' Calls/Sec.')
  end;
end;

end.
